#include "game.h"

// Constructor
GAMEDATA::GAMEDATA()
{
	srand(GetTickCount()); // Seed random number generator

	winner = NO_WINNER; // Nobody has one yet
}

// Init GAMEDATA
void GAMEDATA::init()
{
	// Okay the first thing we're going to do is fill in OUR game board
	// with the "raw data game board" -- This technique is very similar on how
	// "2D tile mapping" is done -- The const array in game.h lays out the shape of our 
	// board and the characters (char) that it should be made up of -- Now we copy to
	// CHAR_INFO array the character AND add a color to be displayed with the char data
	for(int i = 0; i < (BOARD_WDTH * BOARD_HGT); i++)
	{
		screen_buff[i].Char.AsciiChar = (char)game_board[i];
		screen_buff[i].Attributes = FOREGROUND_BLUE; // We'll make 'em blue
	}

	// Get our standard input and output handles
	out_handle = GetStdHandle(STD_OUTPUT_HANDLE);
	in_handle = GetStdHandle(STD_INPUT_HANDLE);

	// Set the rect of where we want to draw to
	scr_rect.Left = X_START;
	scr_rect.Right = X_START + (BOARD_WDTH - 1);
	scr_rect.Top = Y_START;
	scr_rect.Bottom = Y_START + (BOARD_HGT - 1);

	SetConsoleMode(in_handle,ENABLE_MOUSE_INPUT);

	turn = rand()%2; // Randomly pick who's starts

	drawScreen(); // Draw the screen for starters
}

// This draws the screen (draws the tic-tac-toe board)
void GAMEDATA::drawScreen()
{
	COORD size = {BOARD_WDTH , BOARD_HGT}; // The size of the buffer in (width, height) format
	COORD corner = {0, 0}; // Upper left corner to begin writing buffer from							
	
	WriteConsoleOutput(out_handle, screen_buff, size, corner, &scr_rect);
}

// Get the user input (mouse clicks)
void GAMEDATA::getInput()
{
	INPUT_RECORD input_record;
	DWORD events;

	// Get any input
	ReadConsoleInput(in_handle, &input_record, 1, &events);

	if(input_record.EventType == MOUSE_EVENT)
	{
		// If they DID NOT hit the left mouse button, we don't care about the input
		if(input_record.Event.MouseEvent.dwButtonState != FROM_LEFT_1ST_BUTTON_PRESSED)
			return;

		// Get the coordinates the clicked on
		int x_pos = input_record.Event.MouseEvent.dwMousePosition.X;
		int y_pos = input_record.Event.MouseEvent.dwMousePosition.Y;

		// If the clicked in a square on the tic-tac-toe board
		if(clickedSquare(x_pos,y_pos))
		{
			// And if that square was open
			if(squareOpen(x_pos,y_pos))
			{
				// We want to fill it with an 'X' (the human player piece)
				setScreenBuff(x_pos,y_pos,'X');
				nextTurn(); // Computers turn
			}
		}

	} // end of if(input_record.EventType == MOUSE_EVENT)

} // end of void GAMEDATA::getInput()

// Toggle players turn
void GAMEDATA::nextTurn()
{
	turn ^= 1;	// Here we are being TRICKY -- In general, if you can avoid
				// if/switch statements in games, you should -- Now of course
				// for this game, it doesn't make a lick of difference.  But if this 
				// were executed 10,000 times a frame, it would
	
	// The above code is doing this:
/*	if(turn)
		turn = 0;
	else
		turn = 1;	*/
}

// Here we check to see if anybody has won -- If somebody has won, we set "winner" to 
// whoever has won
void GAMEDATA::calcWinner()
{
	if(threeHorz('X') || threeVert('X') || twoDiag('X'))
		winner = HUMAN_WINNER;

	if(threeHorz('O') || threeVert('O') || twoDiag('O'))
		winner = AI_WINNER;

	if(openSqCount() == 0)
		winner = TIE;
}

// This sets the screen buffer (the tic-tac-toe board) to the 'c' (the char) 
// specified at (x,y)
void GAMEDATA::setScreenBuff(int x, int y, char c, bool scrCoords)
{	
	// Okay again more tricks :)
	// Remember we offset our tic-tac-toe board 5 (X_START) from the left and 5 (Y_START)
	// from the top of the console screen (which starts at (0,0))
	// Obviously the "top left coordinate" of our tic-tac-toe board (screen_buff) starts
	// at (0,0) (if you think about it as a 2D array) -- However, where we are displaying 
	// starts at (5,5) -- By moding (using the %) we round the ACTUAL SCREEN COORDINATES of 
	// where the person clicked down to numbers that correspond with our screen buffer array
	// I give an example a few lines down
	if(scrCoords)
	{
		x %= X_START;
		y %= Y_START;
	}
	// Okay example time -- If I click on (5,5) in the console window that is where the 
	// top-left open square of the tic-tac-toe board will appear, but I know that 
	// in the screen_buff array, the top-left square of the tic-tac-toe board is (0,0)
	// Well 5 % 5 = 0 -- So doing that for the x and y component you get (0,0) which 
	// is correct :)

	// Remember we need to treat "screen_buff" as if it were a 2D array
	screen_buff[x + y * BOARD_WDTH].Char.AsciiChar = c;
}

// This returns true if the player "clicked a square" in the tic-tac-toe board
bool GAMEDATA::clickedSquare(int cursorX, int cursorY)
{
	// We start dest_x and dest_y at the screen coordinates of where
	// the first square of the tic-tac-toe board resides
	int dest_x = X_START;
	int dest_y = Y_START;

	// Then we loop through all possible square of the tic-tac-toe board
	// Notice how we add 2 each time -- This is because if you remember our board
	//  | |      // The first open square is (0,0)
	// -----     // (0,1) is a | bar which is NOT an open square
	//  | | 	 // (0,2) is a open square
	// -----
	//  | | 
	for(int i = 0; i < BOARD_HGT; i += 2)
	{
		for(int j = 0; j < BOARD_WDTH; j += 2)
		{
			if((cursorX == dest_x) && (cursorY == dest_y))
				return true;

			dest_x += 2;
		}

		// Once you go across a row start back at the left
		dest_x = X_START;
		dest_y += 2; // Go down to the next row (again we add 2)
					// Look at the tic-tac-toe board above if you are confused :)
	}

	return false;

}

// This returns true if the square is open, false otherwise
// (x,y) can be screen coordinates or "direct screen_buff" offsets 
bool GAMEDATA::squareOpen(int x, int y, bool scrCoords)
{
	if(scrCoords)
	{
		x %= X_START;
		y %= Y_START;
	}

	// Here we cut out the "if statement" -- This evaluatesto true or false
	// and that's what we want to return :)
	return(screen_buff[x + y * BOARD_WDTH].Char.AsciiChar == OPEN);
}


// Okay this methods returns true if there are three "check_char" position horizontally
// in a row on the tic-tac-toe board -- So in case it's not obvious why we're using the
// numbers we are, lets look at the array and it's indexes
//	   The Board		 Array Indexes

/*		X|O|X			 0  1  2  3  4
		-----            5  6  7  8  9
		O|O|X           10 11 12 13 14
		-----			15 16 17 18 19
		O|X|O			20 21 22 23 24
*/
bool GAMEDATA::threeHorz(char check_char)
{
	if((screen_buff[0].Char.AsciiChar == check_char) &&
	   (screen_buff[2].Char.AsciiChar == check_char) &&
	   (screen_buff[4].Char.AsciiChar == check_char))
			return true;

	if((screen_buff[10].Char.AsciiChar == check_char) &&
	   (screen_buff[12].Char.AsciiChar == check_char) &&
	   (screen_buff[14].Char.AsciiChar == check_char))
			return true;

	if((screen_buff[20].Char.AsciiChar == check_char) &&
	   (screen_buff[22].Char.AsciiChar == check_char) &&
	   (screen_buff[24].Char.AsciiChar == check_char))
			return true;

	return false;

}

// Return true if three "check_char" are in a column -- See up above for 
// a board/index layout :)
bool GAMEDATA::threeVert(char check_char)
{
	if((screen_buff[0].Char.AsciiChar == check_char) &&
	   (screen_buff[10].Char.AsciiChar == check_char) &&
	   (screen_buff[20].Char.AsciiChar == check_char))
			return true;

	if((screen_buff[2].Char.AsciiChar == check_char) &&
	   (screen_buff[12].Char.AsciiChar == check_char) &&
	   (screen_buff[22].Char.AsciiChar == check_char))
			return true;

	if((screen_buff[4].Char.AsciiChar == check_char) &&
	   (screen_buff[14].Char.AsciiChar == check_char) &&
	   (screen_buff[24].Char.AsciiChar == check_char))
			return true;

	return false;

}

// This returns true if there are 3 "check_char" diagonally on the board --
// Again check up above a few lines of a board/index map
bool GAMEDATA::twoDiag(char check_char)
{
	if((screen_buff[0].Char.AsciiChar == check_char) &&
	   (screen_buff[12].Char.AsciiChar == check_char) &&
	   (screen_buff[24].Char.AsciiChar == check_char))
			return true;

	if((screen_buff[4].Char.AsciiChar == check_char) &&
	   (screen_buff[12].Char.AsciiChar == check_char) &&
	   (screen_buff[20].Char.AsciiChar == check_char))
			return true;

	return false;

}
	
// Returns the number of open squares on the tic-tac-toe board
int GAMEDATA::openSqCount()
{
	int count = 0;

	for(int i = 0; i < (BOARD_WDTH * BOARD_HGT); i++)
	{
		if(screen_buff[i].Char.AsciiChar == OPEN)
			count++;
	}

	return count;
}



			
