#ifndef GAME_H
#define GAME_H

#include <windows.h>
#include <iostream>
using namespace std;

/*	So what is exactly OOD/OOP?  OOD == Object Oriented Design and OOP == Object Oriented
	Programming.  Basically they're both just buzz words that mean roughly the same thing.
	OOP is simply just a bunch of classes working together to make a program.  Additionally,
	the classes should be "upgradeable".

	Well for this game we have one simple class (one object) -- In more elaborate games
	you would use several classes that can cooperate to make the game.  Ideally, these classes
	should be generic enough to reuse in other games.  When you get into "advanced C++" 
	concepts, such as inheritance and polymorphism, you'll be able to allow the classes to be
	"upgradeable".

	For this tutorial we don't get into any of that, just one really simple class that
	handles most of the game for us.
*/


// We will use these to determine who has one the game
#define NO_WINNER 0
#define HUMAN_WINNER 1
#define AI_WINNER 2
#define TIE 3

// This is our offset from the top left corner of the screen
// The reason we indent is so we can write text above the tic-tac-toe board
#define X_START 5
#define Y_START 5

#define AI_TURN 1 // When "turn" in GAMEDATA is set to one, it's the AI's turn
				 // otherwise it's the human's turn

// The width and height of the tic-tac-toe board -- 
#define BOARD_WDTH 5
#define BOARD_HGT 5
/*  Now this might not be obvious why the width and height are 5
	Look at the following picture and see if it makes more sense.

	X|O|X			// Notice 5 actual characters "wide"
	-----
	O|O|X
	-----
	O|X|O

	// And 5 actual characters in "height"
*/

#define OPEN 32 // 32 in ASCII is the space character (' ')

// Here we define our game board -- If you look up the ASCII representation of the numbers
// (for instance 179) you'll see they correspond to "bars" that effectively create the tic-tac-toe
// board -- Notice how we're going BOARD_WDTH wide and BOARD_HGT tall -- Also notice how 
// we are using a SINGLE ARRAY to hold in essence a DOUBLE ARRAY (or 2-dimensional array as it's
// called)  When making a game it is almost always better to treat a single array as a 2D array
const int game_board[] =
{
	OPEN,179,OPEN,179,OPEN,
	196,197,196,197,196,
	OPEN,179,OPEN,179,OPEN,
	196,197,196,197,196,
	OPEN,179,OPEN,179,OPEN,
};

// This is our GAMEDATA class -- It will embody all that we need for our game
class GAMEDATA
{	
	public:

		// Constructor
		GAMEDATA();

		void init(); // Init game data

		// Data Access ***

			int getTurn() const { return turn; }
			int getWinner() const { return winner; }

		// *** End of Data Access

		// This method sets a character in our tic-tac-toe board
		// (x,y) is the offset into the tic-tac-toe board
		// c is the the "piece" to set (either 'X' or 'O')
		// scrCoords is a boolean saying "Does x and y relate to screen coordinates OR
		// a direct offset into the tic-tac-toe board?" -- If scrCoords equals true
		// then (x,y) is given in screen coordinates (more on this later :)
		void setScreenBuff(int x, int y, char c, bool scrCoords = true);
		
		// Simply draws the tic-tac-toe board to the screen
		void drawScreen();

		// Gets the user input -- The user input is mouse clicks on the screen
		// When a player clicks the screen we are going to get an (x,y) -- This x and y
		// is the SCREEN COORDINATES of where the player clicked
		void getInput();
		
		// Increments to the next persons turn
		void nextTurn();

		// Figures out who won the game
		void calcWinner();

		// Returns true if the player clicked in the SCREEN COORDIANTES (cursorX,cursorY)
		bool clickedSquare(int cursorX, int cursorY);
		bool squareOpen(int cursorX, int cursorY, bool scrCoords = true); // Returns true if
																		 // a square is open
																	    // (ie unoccupied), false
																	   // otherwise
		
		// Returns true if there are 3 'c' (either 'X' or 'O') in a row horizontally
		bool threeHorz(char c);	
		bool threeVert(char c); // Returns true if there are 3 'c' in a row vertically
		bool twoDiag(char c); // Returns true if there are 3 'c' diagonally lined up

		int openSqCount(); // Returns the number of unoccupied squares on the tic-tac-toe board

	private:

		// This is the tic-tac-toe board -- It will contain all the pieces (either 'X' or 'O')
		// and the actual "lines" that make up the board itself
		CHAR_INFO screen_buff[BOARD_WDTH * BOARD_HGT];

		HANDLE out_handle; // Handle to the "screen"
		HANDLE in_handle; // Handle to the "keyboard"

		SMALL_RECT scr_rect; // This rect specifies where on the screen (where in our console
						    // window) should we draw the tic-tac-toe board to

		int turn; // Who's turn is it?

		int winner; // Who has won
};


#endif